@@ -7,17 +7,17 @@
 @版本    :0.1
 '''
 import copy
-from dataclasses import dataclass
-from typing import final
+from re import match
 from sqlalchemy import func
 from sqlalchemy.orm import Session
+from sqlalchemy.orm.attributes import flag_modified
 from sqlalchemy.exc import SQLAlchemyError
 from datetime import datetime
 from sqlalchemy import and_
 from .db.session import createDbSession
 from .db.tables import TblFlowRule, TblFlowJob, TblFlowRecords
 from .flowmodelagent import FlowModelsAgent
-from .datadef import DbConfig, OperResult, FlowCustomRuleNode, QueryParams
+from .datadef import DbConfig, OperResult, QueryParams
 from .error_code import KaruoFlowErrors
 from .db.tables import InitKaruoflowTables
 from .initflow import InitializeFlowsFromConfigure
@@ -175,11 +175,17 @@ class KaruoFlow:
         _total = _common_query.count()
         _skip = (page -1 ) * size
         match_items = _common_query.order_by(TblFlowJob.id.desc()).offset(_skip).limit(size).all()
+        _distinct_ics = set()
+        _filter_items = []
+        for _mi in match_items:
+            if _mi[0].id not in _distinct_ics:
+                _distinct_ics.add(_mi[0].id)
+                _filter_items.append(_mi[0].property2Dict())
         return {
             "total": _total,
             "page": page,
             "size": size,
-            "sets": [mi[0].property2Dict() for mi in match_items]
+            "sets": _filter_items
         }
         
 
@@ -229,6 +235,20 @@ class KaruoFlow:
         _common_query.order_by(TblFlowJob.id.desc())
         return [j.property2Dict() for j in _common_query.all()]
 
+    def QueryJobDecisions(self, jobid: int, stage: int):
+        """
+        查询审批流程指定阶段的决策数据
+        :param jobid int 审批流编号
+        :param stage int 需要查询的阶段
+        """
+        _result = OperResult()
+        _decision_records = self.db_session.query(TblFlowRecords).filter(and_(
+            TblFlowRecords.job_id==jobid,
+            TblFlowRecords.decide_stage==stage
+        )).all()
+        _result.setSuccess([_decision.property2Dict() for _decision in _decision_records])
+
+        return _result
 
 
     # ======================= operations
@@ -334,6 +354,25 @@ class KaruoFlow:
 
         return _result
 
+    def QueryJobViaRelationPrimaryKey(self, catalog: str, prikey: int, subflow: str = None) -> dict:
+        """
+        根据关联主键，查询审批任务
+        """
+        _result = OperResult()
+        _query = self.db_session.query(TblFlowJob).filter(and_(
+            TblFlowJob.catalog == catalog,
+            TblFlowJob.rel_prikey == prikey
+        ))
+        if subflow:
+            _query = _query.filter(TblFlowJob.subflow==subflow)
+        _job_item = _query.order_by(TblFlowJob.id.desc()).first()
+        if _job_item:
+            _result.setSuccess(_job_item.property2Dict())
+        else:
+            _result.setNotExists()
+
+        return _result
+
     def ApplyCustom(self, userid: str, catalog: str, desc: str, rules:list, subflow: str="", ext_data:dict={}, rel_prikey=0):
         """
         发起自由流程
@@ -480,6 +519,31 @@ class KaruoFlow:
         else:
             ret_code = KaruoFlowErrors.ERR_DATA_NOT_FOUND
         return ret_code
+    
+    def ExamineCoherent(self, jobid: int, user_id: str, agree: bool, desc: str):
+        """
+        申请人连续审批
+        :param jobid 申请编号
+        :param user_id 审批者编号
+        :param agree 通过或拒绝
+        :param desc 备注信息
+        """
+        ret_code = KaruoFlowErrors.ERR_UNKOWN
+        apply_job = self.db_session.query(TblFlowJob).filter(TblFlowJob.id==jobid).first()
+        if apply_job:
+            if user_id in apply_job.reviewer:
+                if apply_job.closed == '0':
+                    if agree:
+                        ret_code = self.flows_agent.AgreeJobFlowCoherent(apply_job, user_id, desc)
+                    else:
+                        ret_code = self.flows_agent.RefuseJobFlow(apply_job, user_id, desc)
+                else:
+                    ret_code = KaruoFlowErrors.ERR_FLOW_CLOSED
+            else:
+                ret_code = KaruoFlowErrors.ERR_FLOW_OWNER_INVALID
+        else:
+            ret_code = KaruoFlowErrors.ERR_DATA_NOT_FOUND
+        return ret_code
 
     def Examine(self, jobid:int, user_id:str, agree:bool, desc:str):
         """
@@ -506,14 +570,14 @@ class KaruoFlow:
             ret_code = KaruoFlowErrors.ERR_DATA_NOT_FOUND
         return ret_code
 
-    def ExamineStage(self, jobid: int, user_id:str, agree:bool, desc:str):
+    def ExamineStageCoherent(self, jobid: int, user_id: str, agree: bool, desc: str, autograph: str): 
         """
-        对申请进行阶段审批
-        ** 该函数支持会签操作
+        对申请进行阶段审批（同一审批人进行连续审批）
         :param jobid 申请编号
         :param user_id 审批者编号
         :param agree 通过或拒绝
         :param desc 备注信息
+        :param autograph 手写签名信息
         """
         ret_code = KaruoFlowErrors.ERR_UNKOWN
         apply_job = self.db_session.query(TblFlowJob).filter(TblFlowJob.id==jobid).first()
@@ -521,7 +585,33 @@ class KaruoFlow:
             if user_id in apply_job.reviewer:
                 if apply_job.closed == '0':
                     if agree:
-                        ret_code = self.flows_agent.AgreeJobFlow_V2(apply_job, user_id, desc)
+                        ret_code = self.flows_agent.AgreeJobFlow_V2Coherent(apply_job, user_id, desc, autograph)
+                    else:
+                        ret_code = self.flows_agent.RefuseJobFlow_V2(apply_job, user_id, desc)
+                else: ret_code = KaruoFlowErrors.ERR_FLOW_CLOSED
+            else: ret_code = KaruoFlowErrors.ERR_FLOW_OWNER_INVALID
+        else: ret_code = KaruoFlowErrors.ERR_DATA_NOT_FOUND
+
+        return ret_code
+
+
+    def ExamineStage(self, jobid: int, user_id:str, agree:bool, desc:str, autograph:str):
+        """
+        对申请进行阶段审批
+        ** 该函数支持会签操作
+        :param jobid 申请编号
+        :param user_id 审批者编号
+        :param agree 通过或拒绝
+        :param desc 备注信息
+        :param autograph 手写签名信息
+        """
+        ret_code = KaruoFlowErrors.ERR_UNKOWN
+        apply_job = self.db_session.query(TblFlowJob).filter(TblFlowJob.id==jobid).first()
+        if apply_job:
+            if user_id in apply_job.reviewer:
+                if apply_job.closed == '0':
+                    if agree:
+                        ret_code = self.flows_agent.AgreeJobFlow_V2(apply_job, user_id, desc, autograph)
                     else:
                         ret_code = self.flows_agent.RefuseJobFlow_V2(apply_job, user_id, desc)
                 else: ret_code = KaruoFlowErrors.ERR_FLOW_CLOSED
@@ -554,4 +644,28 @@ class KaruoFlow:
             else: _result.setErrorInfor(KaruoFlowErrors.ERR_FLOW_CLOSED, KaruoFlowErrors.errorMsg(KaruoFlowErrors.ERR_FLOW_CLOSED))
         else: _result.setErrorInfor(KaruoFlowErrors.ERR_DATA_NOT_FOUND, KaruoFlowErrors.errorMsg(KaruoFlowErrors.ERR_DATA_NOT_FOUND))
         
+        return _result
+
+    def ResetReviewers(self, jobid: int, current_stage: int, reviewers: list):
+        '''
+        重置当前阶段审核人员
+        '''
+        _result = OperResult()
+        apply_job = self.db_session.query(TblFlowJob).filter(TblFlowJob.id==jobid).first()
+        if apply_job:
+            if apply_job.stage == current_stage:
+                try:
+                    apply_job.reviewer = reviewers
+                    flag_modified(apply_job, "reviewer")
+                    self.db_session.commit()
+                    _result.setSuccess()
+                except SQLAlchemyError as e:
+                    self.db_session.rollback()
+                    _result.setErrorInfor(KaruoFlowErrors.ERR_DB_EXCEPTION, str(e))
+                finally:
+                    self.db_session.close()
+            else:
+                _result.setErrorInfor(KaruoFlowErrors.ERR_FLOW_STAGE_ERROR, KaruoFlowErrors.errorMsg(KaruoFlowErrors.ERR_FLOW_STAGE_ERROR))
+        else: 
+            _result.setErrorInfor(KaruoFlowErrors.ERR_DATA_NOT_FOUND, KaruoFlowErrors.errorMsg(KaruoFlowErrors.ERR_DATA_NOT_FOUND))
         return _result
\ No newline at end of file
