@@ -50,6 +50,11 @@ class InitializeEnvironmentTest(unittest.TestCase):
         self.assertEqual(KaruoFlowErrors.SUCCESS, ret_code)
         self.assertLess(0, job_id)
 
+    def test_applyflow_instance1(self):
+        ret_code, job_id = self.flow_app.Apply("sunrui", "dn_confirm_meeting", "test demo")
+        print(ret_code)
+        print(job_id)
+
     def test_applylist_indoing(self):
         res = self.flow_app.QueryApplyListInDoing("caimmy", None)
         pprint(res)
@@ -131,7 +136,7 @@ class InitializeEnvironmentTest(unittest.TestCase):
             self.flow_app.Examine(m[0].id, "caimmy", True, "adf")
 
     def test_query_jobinfor(self):
-        t = self.flow_app.QueryJob(301)
+        t = self.flow_app.QueryJob(443)
         pprint(t.data)
 
     def test_create_custom_flow(self):
@@ -165,7 +170,7 @@ class InitializeEnvironmentTest(unittest.TestCase):
         # qp.setMatches({"apply_user": "shenxiaolin"})
         tp = QueryParams()
         tp.setContains({"description": "爱上"})
-        s = self.flow_app.QueryApplyDecidedByUser("shenxiaolin", record_condition=tp)
+        s = self.flow_app.QueryApplyDecidedByUser("long", record_condition=None)
         pprint(s)
         print(len(s.get("sets")))
 
@@ -189,18 +194,29 @@ class InitializeEnvironmentTest(unittest.TestCase):
         _catalog = "caiwu"
         ret_code, job_id = self.flow_app.ApplyCustom("caimmy", _catalog, "custom apply test", [
             FlowCustomRuleNode(_catalog, ['caimmy']),
-            FlowCustomRuleNode(_catalog, ['caimmy', 'long'], 'chuchai', node_label='middle node'),
-            FlowCustomRuleNode(_catalog, ['liangchaowei', 'guldegang'], method="joint")
+            FlowCustomRuleNode(_catalog, ['liliqun', 'long'], 'ac', node_label="aaabbb"),
+            FlowCustomRuleNode(_catalog, ['caimmy', 'long'], 'chuchai', node_label='middle node', method="joint"),
+            FlowCustomRuleNode(_catalog, ['liangchaowei', 'guodegang', "caimmy"], method="joint")
         ], subflow='chuchai')
         self.assertEqual(ret_code, KaruoFlowErrors.SUCCESS)
+        print("job_id", job_id)
         self.assertLess(0, job_id)
 
+    def test_coherent_examine(self):
+        """
+        测试普通审批
+        """
+        ret_code = self.flow_app.ExamineCoherent(396, "long", True, "demo test")
+        print(ret_code)
+
     def test_examine_stage(self):
         """
         支持会签的审批
         """
-        ret_code = self.flow_app.ExamineStage(302, "guldegang", True, "joint agree")
+        # ret_code = self.flow_app.ExamineStage(445, "caimmy", True, "joint agree", "http://www.baidu.com")
+        ret_code = self.flow_app.ExamineStageCoherent(451, "long", True, "joint agree", "http://www.baidu.com")
         print(ret_code)
+        print(self.flow_app.QueryJob(434))
 
     def test_transfer_nextstage(self):
         '''
@@ -217,6 +233,27 @@ class InitializeEnvironmentTest(unittest.TestCase):
         print("-- 3 --")
         pprint(t.data.get("job"))
 
+    def testQueryStageDecisions(self):
+        """
+        测试阶段审批记录
+        """
+        t = self.flow_app.QueryJobDecisions(426, 2)
+        pprint(t)
+
+    def testSetStageDecisionUsers(self):
+        t = self.flow_app.ResetReviewers(456, 1, ['sunwukong', 'zhubajie'])
+        pprint(t)
+
+
+    def testQueryJobByPrikey(self):
+        t = self.flow_app.QueryJobViaRelationPrimaryKey("dn_confirm_meeting", 15)
+        pprint(t)
+
+        m = self.flow_app.QueryJobViaRelationPrimaryKey("dn_hold_meeting", 28, "office_work")
+        pprint(m)
+
+        m = self.flow_app.QueryJobViaRelationPrimaryKey("dn_hold_meeting", 1, "office_work")
+        print(m)
 
 if "__main__" == __name__:
     unittest.main()
\ No newline at end of file
